package App::lupapona;

our $VERSION = 1.07;

=head1 NAME

App::lupapona - a simple static Gemini server

=head1 DESCRIPTION

This is a stub module, see F<script/lupa-pona> for details of the app.

=head1 AUTHOR

Alex Schroeder

=head1 LICENSE

GNU Affero General Public License

=cut

1;
